local RC = LibStub("AceAddon-3.0"):GetAddon("Raid Check")
local L = LibStub("AceLocale-3.0"):GetLocale("Raid Check", true)
local RC_UI = LibStub("AceGUI-3.0")

local RC_Main = nil
local RC_Settings = nil
local RC_Tabs = nil
local _activeTab = "RC_Main"

-- release everything when the UI is closed
local function onMainFrameClose(widget)
	RC_UI:Release(widget)
	RC["ReleaseTab" .. _activeTab](RC)

	RC_Main = nil
	RC_Settings = nil
	RC_Tabs = nil
end


-- This will create the Main Tab of the addon, where the players will be listed with their current status
function RC:MainTab(container)
	-- Load the Data first, so we can use it while building the Player Table
	RC:loadData()

	-- Group to hold the three tab buttons, for easy movement and organization
	local buttonGroup = RC_UI:Create("SimpleGroup")
	buttonGroup:SetWidth(RC.GrpWidth);
	buttonGroup:SetPoint("TOPLEFT")
	buttonGroup:SetLayout("Flow")

	-- Announce Button
	local announceBtn = RC_UI:Create("Button")
	announceBtn:SetWidth(RC.ButtonWidth * 1.12)
	announceBtn:SetHeight(RC.ButtonHeight)
	announceBtn:SetPoint("TOPLEFT", 0, -5)
	announceBtn:SetText(L["announce"])
	announceBtn:SetCallback("OnClick", function()
		RC:Announce();
	end )
	announceBtn:SetCallback("OnEnter", function()
		GameTooltip:SetOwner(UIParent, "ANCHOR_CURSOR");
		GameTooltip:SetText(L["announcetxt"]);
		GameTooltip:Show();
	end)
	announceBtn:SetCallback("OnLeave", function(self)
		GameTooltip:Hide()
	end)
	buttonGroup:AddChild(announceBtn)

	-- Whisper Button
	local whisperBtn = RC_UI:Create("Button")
	whisperBtn:SetWidth(RC.ButtonWidth)
	whisperBtn:SetHeight(RC.ButtonHeight)
	whisperBtn:SetPoint("TOPLEFT", (RC.ButtonWidth + 5), -5)
	whisperBtn:SetText(L["whisper"])
	whisperBtn:SetCallback("OnClick", function()
		RC:Whisper(-1);
	end)
	whisperBtn:SetCallback("OnEnter", function(self)
		GameTooltip:SetOwner(UIParent, "ANCHOR_CURSOR");
		GameTooltip:SetText(L["whispertxt"]);
		GameTooltip:Show();
	end)
	whisperBtn:SetCallback("OnLeave", function(self)
		GameTooltip:Hide()
	end)
	buttonGroup:AddChild(whisperBtn)

	-- Refresh Button
	local refreshBtn = RC_UI:Create("Button")
	refreshBtn:SetWidth(RC.ButtonWidth)
	refreshBtn:SetHeight(RC.ButtonHeight)
	refreshBtn:SetPoint("TOPLEFT", (RC.ButtonWidth + 5) * 2, -5)
	refreshBtn:SetText(L["refresh"])
	refreshBtn:SetCallback("OnClick", function()
		RC:Hide()
		RC:OpenMain()
		collectgarbage("collect");
	end)
	refreshBtn:SetCallback("OnEnter", function(self)
		GameTooltip:SetOwner(UIParent, "ANCHOR_CURSOR");
		GameTooltip:SetText(L["refreshtxt"]);
		GameTooltip:Show();
	end)
	refreshBtn:SetCallback("OnLeave", function(self)
		GameTooltip:Hide();
	end)
	buttonGroup:AddChild(refreshBtn)

	-- Group to hold the Primary Stat Status
	local statusGroup = RC_UI:Create("SimpleGroup")
	if self.db.char.Allow.PrimaryStat then
		statusGroup:SetWidth(RC.GrpWidth);
		statusGroup:SetHeight(15)
		statusGroup:SetPoint("TOPLEFT", 8, -35) ;
		statusGroup:SetLayout("Flow")

		local PSB = {};
		for i = 1, table.getn(RC.PrimaryStatBuffs) do
			local name, _, _, _, _, _, _ = GetSpellInfo(RC.PrimaryStatBuffs[i][1])
			PSB[i] = RC_UI:Create("InteractiveLabel");
			PSB[i]:SetText(name)
			PSB[i]:SetColor(1,0,0)  -- Default the color to Red, we only change this if we have the class present

			if i == 1 then
				local className = GetClassInfo(8)
				-- If a Mage is present in the group then we change the color to yellow
				if RC.ClassInGroup[8] == true then
					PSB[i]:SetColor(1,1,0)
					PSB[i]:SetCallback("OnEnter", function(self)
						GameTooltip:SetOwner(UIParent, "ANCHOR_CURSOR");
						GameTooltip:SetText(className..": "..RC.Mages);
						GameTooltip:Show();
					end)
				else
					PSB[i]:SetCallback("OnEnter", function(self)
						GameTooltip:SetOwner(UIParent, "ANCHOR_CURSOR");
						GameTooltip:SetText(L["no"].." "..className);
						GameTooltip:Show();
					end)
				end
				PSB[i]:SetWidth(RC.SpellWidth)
			elseif i == 2 then
				-- If a Warrior is present in the group then we change the color to yellow
				local className = GetClassInfo(1)
				if RC.ClassInGroup[1] == true then
					PSB[i]:SetColor(1,1,0)
					PSB[i]:SetCallback("OnEnter", function(self)
						GameTooltip:SetOwner(UIParent, "ANCHOR_CURSOR");
						GameTooltip:SetText(className..": "..RC.Warriors);
						GameTooltip:Show();
					end)
				else
					PSB[i]:SetCallback("OnEnter", function(self)
						GameTooltip:SetOwner(UIParent, "ANCHOR_CURSOR");
						GameTooltip:SetText(L["no"].." "..className);
						GameTooltip:Show();
					end)
				end
				PSB[i]:SetWidth(RC.SpellWidth)
			else
				-- If a Priest is present in the group then we change the color to yellow
				local className = GetClassInfo(5)
				if RC.ClassInGroup[5] == true then
					PSB[i]:SetColor(1,1,0)
					PSB[i]:SetCallback("OnEnter", function(self)
						GameTooltip:SetOwner(UIParent, "ANCHOR_CURSOR");
						GameTooltip:SetText(className..": "..RC.Priests);
						GameTooltip:Show();
					end)
				else
					PSB[i]:SetCallback("OnEnter", function(self)
						GameTooltip:SetOwner(UIParent, "ANCHOR_CURSOR");
						GameTooltip:SetText(L["no"].." "..className);
						GameTooltip:Show();
					end)
				end
				PSB[i]:SetWidth(RC.SpellWidth + 25)
			end

			-- If the buff is present then we change the color to Green, overriding the Yellow or Red color
			if RC.PrimaryStatBuffs[i][2] == true then
				PSB[i]:SetColor(0,1,0)
				PSB[i]:SetCallback("OnEnter", function(self)
				end)
			end

			PSB[i]:SetCallback("OnLeave", function(self)
				GameTooltip:Hide();
			end)

			statusGroup:AddChild(PSB[i]);
		end
	end

	-- Group to Hold the Damage Modifiers
	local dmgModsGroup = RC_UI:Create("SimpleGroup")
	if self.db.char.Allow.DmgModDeBuff then
		dmgModsGroup:SetWidth(RC.GrpWidth);
		dmgModsGroup:SetHeight(15)
		dmgModsGroup:SetPoint("TOPLEFT", 8, -35) ;
		dmgModsGroup:SetLayout("Flow")

		local dmgTitle = RC_UI:Create("Label")
		dmgTitle:SetText(L["dmgTitle"])
		dmgTitle:SetWidth(RC.PlayerWidth + 5)
		dmgModsGroup:AddChild(dmgTitle)

		-- Demon Hunter gives this Magic Damage Debuff
		local magicDmgLbl = RC_UI:Create("InteractiveLabel")
		magicDmgLbl:SetText(L["magic"])
		if RC.ClassInGroup[12] == true then
			magicDmgLbl:SetColor(0,1,0)
		else
			magicDmgLbl:SetColor(1,0,0)

			magicDmgLbl:SetCallback("OnEnter", function(self)
				GameTooltip:SetOwner(UIParent, "ANCHOR_CURSOR");
				local className = GetClassInfo(12)
				GameTooltip:SetText(L["no"].." "..className);
				GameTooltip:Show();
			end)
			magicDmgLbl:SetCallback("OnLeave", function(self)
				GameTooltip:Hide();
			end)
		end
		magicDmgLbl:SetWidth(RC.FlFoRuWidth)
		dmgModsGroup:AddChild(magicDmgLbl)

		-- Monk gives this Physical Damage Debuff
		local physDmgLbl = RC_UI:Create("InteractiveLabel")
		physDmgLbl:SetText(L["physical"])
		if RC.ClassInGroup[10] == true then
			physDmgLbl:SetColor(0,1,0)
		else
			physDmgLbl:SetColor(1,0,0)

			physDmgLbl:SetCallback("OnEnter", function(self)
				GameTooltip:SetOwner(UIParent, "ANCHOR_CURSOR");
				local className = GetClassInfo(10)
				GameTooltip:SetText(L["no"].." "..className);
				GameTooltip:Show();
			end)
			physDmgLbl:SetCallback("OnLeave", function(self)
				GameTooltip:Hide();
			end)
		end
		physDmgLbl:SetWidth(RC.FlFoRuWidth+25)
		dmgModsGroup:AddChild(physDmgLbl)
	end

	-- Group to Hold the Utilities
	local utilGroup = RC_UI:Create("SimpleGroup")
	if self.db.char.Allow.Utility then
		utilGroup:SetWidth(RC.GrpWidth);
		utilGroup:SetHeight(30)
		utilGroup:SetPoint("TOPLEFT", 8, -35) ;
		utilGroup:SetLayout("Flow")


		-- 3:Hunter 11:Druid
		local dispelEnrageLbl = RC_UI:Create("InteractiveLabel")
		dispelEnrageLbl:SetText(L["dispelEnrage"])
		-- If we have both Hunter and Druid then green
		if RC.ClassInGroup[3] == true and RC.ClassInGroup[11] == true then
			dispelEnrageLbl:SetColor(0,1,0)
		elseif RC.ClassInGroup[3] == true or RC.ClassInGroup[11] == true then  --Hunters, no Druids
			dispelEnrageLbl:SetColor(1,1,0)
			local missingClasses = L["no"];
			if RC.ClassInGroup[3] == false then
				local className = GetClassInfo(3)
				missingClasses = missingClasses.." "..className
			end
			if RC.ClassInGroup[11] == false then
				local className = GetClassInfo(11)
				missingClasses = missingClasses.." "..className
			end
			dispelEnrageLbl:SetCallback("OnEnter", function(self)
				GameTooltip:SetOwner(UIParent, "ANCHOR_CURSOR");
				GameTooltip:SetText(missingClasses);
				GameTooltip:Show();
			end)
			dispelEnrageLbl:SetCallback("OnLeave", function(self)
				GameTooltip:Hide();
			end)
		else
			dispelEnrageLbl:SetColor(1,0,0)
			dispelEnrageLbl:SetCallback("OnEnter", function(self)
				GameTooltip:SetOwner(UIParent, "ANCHOR_CURSOR");
				local className = GetClassInfo(3)
				local className2 = GetClassInfo(11)
				GameTooltip:SetText(L["no"].." "..className.." - "..className2);
				GameTooltip:Show();
			end)
			dispelEnrageLbl:SetCallback("OnLeave", function(self)
				GameTooltip:Hide();
			end)
		end
		dispelEnrageLbl:SetWidth(RC.SpellWidth)
		utilGroup:AddChild(dispelEnrageLbl)

		-- 5:Priest, 7:Shaman, 8:Mage, 9:Warlock, 12:Demon Hunter (with so many classes, initialy going to just use colors, no tooltips)
		local purgeMagicLbl = RC_UI:Create("InteractiveLabel")
		purgeMagicLbl:SetText(L["purgeMagic"])
		if RC.ClassInGroup[5] == true and RC.ClassInGroup[7] == true and RC.ClassInGroup[8] == true and RC.ClassInGroup[9] == true and RC.ClassInGroup[12] == true then
			purgeMagicLbl:SetColor(0,1,0)
		elseif RC.ClassInGroup[5] == true or RC.ClassInGroup[7] == true or RC.ClassInGroup[8] == true or RC.ClassInGroup[9] == true or RC.ClassInGroup[12] == true then
			purgeMagicLbl:SetColor(1,1,0)
			local missingClasses = L["no"];
			if RC.ClassInGroup[5] == false then
				local className = GetClassInfo(5)
				missingClasses = missingClasses.." "..className.." -"
			end
			if RC.ClassInGroup[7] == false then
				local className = GetClassInfo(7)
				missingClasses = missingClasses.." "..className.." -"
			end
			if RC.ClassInGroup[8] == false then
				local className = GetClassInfo(8)
				missingClasses = missingClasses.." "..className.." -"
			end
			if RC.ClassInGroup[9] == false then
				local className = GetClassInfo(9)
				missingClasses = missingClasses.." "..className.." -"
			end
			if RC.ClassInGroup[12] == false then
				local className = GetClassInfo(12)
				missingClasses = missingClasses.." "..className
			end
			purgeMagicLbl:SetCallback("OnEnter", function(self)
				GameTooltip:SetOwner(UIParent, "ANCHOR_CURSOR");
				GameTooltip:SetText(missingClasses);
				GameTooltip:Show();
			end)
			purgeMagicLbl:SetCallback("OnLeave", function(self)
				GameTooltip:Hide();
			end)
		else
			purgeMagicLbl:SetColor(1,0,0)
			purgeMagicLbl:SetCallback("OnEnter", function(self)
				GameTooltip:SetOwner(UIParent, "ANCHOR_CURSOR");
				local className = GetClassInfo(5)
				local className2 = GetClassInfo(7)
				local className3 = GetClassInfo(8)
				local className4 = GetClassInfo(9)
				local className5 = GetClassInfo(12)
				GameTooltip:SetText(L["no"].." "..className.." - "..className2.." - "..className3.." - "..className4.." - "..className5);
				GameTooltip:Show();
			end)
			purgeMagicLbl:SetCallback("OnLeave", function(self)
				GameTooltip:Hide();
			end)
		end
		purgeMagicLbl:SetWidth(RC.SpellWidth)
		utilGroup:AddChild(purgeMagicLbl)

		-- 7:Shaman, 8:Mage, 11:Druid
		local removeCurseLbl = RC_UI:Create("InteractiveLabel")
		removeCurseLbl:SetText(L["removeCurse"])
		if RC.ClassInGroup[7] == true and RC.ClassInGroup[8] == true and RC.ClassInGroup[11] == true then
			removeCurseLbl:SetColor(0,1,0)
		elseif RC.ClassInGroup[7] == true or RC.ClassInGroup[8] == true or RC.ClassInGroup[11] == true then
			removeCurseLbl:SetColor(1,1,0)
			local missingClasses = L["no"];
			if RC.ClassInGroup[7] == false then
				local className = GetClassInfo(7)
				missingClasses = missingClasses.." "..className.." -"
			end
			if RC.ClassInGroup[8] == false then
				local className = GetClassInfo(8)
				missingClasses = missingClasses.." "..className.." -"
			end
			if RC.ClassInGroup[11] == false then
				local className = GetClassInfo(11)
				missingClasses = missingClasses.." "..className
			end
			removeCurseLbl:SetCallback("OnEnter", function(self)
				GameTooltip:SetOwner(UIParent, "ANCHOR_CURSOR");
				GameTooltip:SetText(missingClasses);
				GameTooltip:Show();
			end)
			removeCurseLbl:SetCallback("OnLeave", function(self)
				GameTooltip:Hide();
			end)
		else
			removeCurseLbl:SetColor(1,0,0)
			removeCurseLbl:SetCallback("OnEnter", function(self)
				GameTooltip:SetOwner(UIParent, "ANCHOR_CURSOR");
				local className = GetClassInfo(7)
				local className2 = GetClassInfo(8)
				local className3 = GetClassInfo(11)
				GameTooltip:SetText(L["no"].." "..className.." - "..className2.." - "..className3);
				GameTooltip:Show();
			end)
			removeCurseLbl:SetCallback("OnLeave", function(self)
				GameTooltip:Hide();
			end)
		end
		removeCurseLbl:SetWidth(RC.SpellWidth)
		utilGroup:AddChild(removeCurseLbl)

		-- 2:Paladin, 5:Priest, 10:Monk
		local removeDiseaseLbl = RC_UI:Create("InteractiveLabel")
		removeDiseaseLbl:SetText(L["removeDisease"])
		if RC.ClassInGroup[2] == true and RC.ClassInGroup[5] == true and RC.ClassInGroup[10] == true then
			removeDiseaseLbl:SetColor(0,1,0)
		elseif RC.ClassInGroup[2] == true or RC.ClassInGroup[5] == true or RC.ClassInGroup[10] == true then
			removeDiseaseLbl:SetColor(1,1,0)
			local missingClasses = L["no"];
			if RC.ClassInGroup[2] == false then
				local className = GetClassInfo(2)
				missingClasses = missingClasses.." "..className.." -"
			end
			if RC.ClassInGroup[5] == false then
				local className = GetClassInfo(5)
				missingClasses = missingClasses.." "..className.." -"
			end
			if RC.ClassInGroup[10] == false then
				local className = GetClassInfo(10)
				missingClasses = missingClasses.." "..className
			end
			removeDiseaseLbl:SetCallback("OnEnter", function(self)
				GameTooltip:SetOwner(UIParent, "ANCHOR_CURSOR");
				GameTooltip:SetText(missingClasses);
				GameTooltip:Show();
			end)
			removeDiseaseLbl:SetCallback("OnLeave", function(self)
				GameTooltip:Hide();
			end)
		else
			removeDiseaseLbl:SetColor(1,0,0)
			removeDiseaseLbl:SetCallback("OnEnter", function(self)
				GameTooltip:SetOwner(UIParent, "ANCHOR_CURSOR");
				local className = GetClassInfo(2)
				local className2 = GetClassInfo(5)
				local className3 = GetClassInfo(10)
				GameTooltip:SetText(L["no"].." "..className.." - "..className2.." - "..className3);
				GameTooltip:Show();
			end)
			removeDiseaseLbl:SetCallback("OnLeave", function(self)
				GameTooltip:Hide();
			end)
		end
		removeDiseaseLbl:SetWidth(RC.SpellWidth)
		utilGroup:AddChild(removeDiseaseLbl)

		-- 2:Paladin, 10:MOnk, 11:Druid
		local removePoisonLbl = RC_UI:Create("InteractiveLabel")
		removePoisonLbl:SetText(L["removePoison"])
		if RC.ClassInGroup[2] == true and RC.ClassInGroup[10] == true and RC.ClassInGroup[11] == true then
			removePoisonLbl:SetColor(0,1,0)
		elseif RC.ClassInGroup[2] == true or RC.ClassInGroup[10] == true or RC.ClassInGroup[11] == true then
			removePoisonLbl:SetColor(1,1,0)
			local missingClasses = L["no"];
			if RC.ClassInGroup[2] == false then
				local className = GetClassInfo(2)
				missingClasses = missingClasses.." "..className.." -"
			end
			if RC.ClassInGroup[10] == false then
				local className = GetClassInfo(10)
				missingClasses = missingClasses.." "..className.." -"
			end
			if RC.ClassInGroup[11] == false then
				local className = GetClassInfo(11)
				missingClasses = missingClasses.." "..className
			end
			removePoisonLbl:SetCallback("OnEnter", function(self)
				GameTooltip:SetOwner(UIParent, "ANCHOR_CURSOR");
				GameTooltip:SetText(missingClasses);
				GameTooltip:Show();
			end)
			removePoisonLbl:SetCallback("OnLeave", function(self)
				GameTooltip:Hide();
			end)
		else
			removePoisonLbl:SetColor(1,0,0)
			removePoisonLbl:SetCallback("OnEnter", function(self)
				GameTooltip:SetOwner(UIParent, "ANCHOR_CURSOR");
				local className = GetClassInfo(2)
				local className2 = GetClassInfo(10)
				local className3 = GetClassInfo(11)
				GameTooltip:SetText(L["no"].." "..className.." - "..className2.." - "..className3);
				GameTooltip:Show();
			end)
			removePoisonLbl:SetCallback("OnLeave", function(self)
				GameTooltip:Hide();
			end)
		end
		removePoisonLbl:SetWidth(RC.SpellWidth)
		utilGroup:AddChild(removePoisonLbl)
	end

	-- Group to hold the heading labels for the list of players
	local headGroup = RC_UI:Create("SimpleGroup")
	headGroup:SetWidth(RC.GrpWidth);
	headGroup:SetHeight(15)
	headGroup:SetPoint("TOPLEFT", 8, -35) ;
	headGroup:SetLayout("Flow")

	-- Player heading
	local playLbl = RC_UI:Create("Label")
	playLbl:SetText(L["player"])
	playLbl:SetWidth(RC.PlayerWidth)
	playLbl:SetPoint("TOPLEFT", 5, -2)
	headGroup:AddChild(playLbl)

	-- Flask heading
	local flaskLbl = RC_UI:Create("Label")
	flaskLbl:SetText(L["flask"])
	flaskLbl:SetWidth(RC.FlFoRuWidth)
	flaskLbl:SetPoint("TOPLEFT", (5 + RC.PlayerWidth + RC.Spacing), -2)
	headGroup:AddChild(flaskLbl)

	-- Food heading
	local foodLbl = RC_UI:Create("Label")
	foodLbl:SetText(L["food"])
	foodLbl:SetWidth(RC.FlFoRuWidth)
	foodLbl:SetPoint("TOPLEFT", (5 + RC.PlayerWidth + (RC.Spacing * 2) + RC.FlFoRuWidth), -2)
	headGroup:AddChild(foodLbl)

	-- Rune heading
	if(self.db.char.Show.Rune) then
		local runeLbl = RC_UI:Create("Label")
		runeLbl:SetText(L["rune"])
		runeLbl:SetWidth(RC.FlFoRuWidth)
		runeLbl:SetPoint("TOPLEFT", (5 + RC.PlayerWidth + (RC.Spacing * 3) + (RC.FlFoRuWidth * 2)), -2)
		headGroup:AddChild(runeLbl)
	end

	-- Vantus Rune heading
	if RC.VantusRuneCheckEnabled then
		local vantusRuneLbl = RC_UI:Create("Label")
		vantusRuneLbl:SetText(L["vantusrune"])
		vantusRuneLbl:SetWidth(RC.FlFoRuWidth)
		vantusRuneLbl:SetPoint("TOPLEFT", (5 + RC.PlayerWidth + (RC.Spacing * 3) + (RC.FlFoRuWidth * 2)), -2)
		headGroup:AddChild(vantusRuneLbl)
	end

	-- Scrolling container to hold the list of players and the status of their Flask/Food/Rune
	local scrollcontainer = RC_UI:Create("SimpleGroup") -- "InlineGroup" is also good
	scrollcontainer:SetFullWidth(true)
	scrollcontainer:SetFullHeight(true)
	scrollcontainer:SetLayout("Fill")

	scroll = RC_UI:Create("ScrollFrame")
	scroll:SetLayout("Flow")
	scrollcontainer:AddChild(scroll)

	local numPlayers = GetNumGroupMembers()
	if numPlayers == 0 then
		numPlayers = 1
	end
	playerBar = {}
	nameTbl = {}
	flaskTbl = {}
	foodTbl = {}
	runeTbl = {}
	vantusruneTbl = {}

	for i=1,numPlayers do
		local posY = -2-(15*(i-1))

		RC:PlayerRow(posY, i)
	end

	-- This adds each Group of controls to the container, or tab, we add them all in one place for ease of manipulation
	container:AddChild(buttonGroup)
	if self.db.char.Allow.PrimaryStat then container:AddChild(statusGroup) end
	if self.db.char.Allow.DmgModDeBuff then container:AddChild(dmgModsGroup) end
	if self.db.char.Allow.Utility then container:AddChild(utilGroup) end
	container:AddChild(headGroup)
	container:AddChild(scrollcontainer)
end

function RC:PlayerRow(positionY, row)
	local flStatus, flColor, foStatus, foColor, ruStatus, ruColor
	local allowCtrlWhisper = self.db.char.Whisper.OnControl
	_, _, subgroup, _, _, _, _, _, _, _, _ = GetRaidRosterInfo(row);

	if ((subgroup == 7 or subgroup == 8) and self.db.char.IgnoreGroup7_8) then
		return
	end

	if(row%2 == 0) then
		playerBar[row] = RC_UI:Create("RCSimpleGroup_LG")
	else
		playerBar[row] = RC_UI:Create("RCSimpleGroup_DG")
	end
	playerBar[row]:SetHeight(15)
	playerBar[row]:SetPoint("TOPLEFT", 0, positionY)
	playerBar[row]:SetLayout("Flow")

	nameTbl[row] = RC_UI:Create("InteractiveLabel")
	nameTbl[row]:SetText(RC:PlayerName(row))
	nameTbl[row]:SetColor(unpack(RC:SetClassColor(row)))
	nameTbl[row]:SetPoint("TOPLEFT", 2, -2)
	nameTbl[row]:SetWidth(RC.PlayerWidth)
	nameTbl[row]:SetCallback("OnClick", function(self, button)
		if(IsControlKeyDown() and allowCtrlWhisper) then
			RC:Whisper(row);
		end
	end)
	playerBar[row]:AddChild(nameTbl[row])

	flStatus, flColor = RC:SetFlFoRu(row, 3)
	flaskTbl[row] = RC_UI:Create("InteractiveLabel")
	flaskTbl[row]:SetText(flStatus)
	flaskTbl[row]:SetColor(unpack(flColor))
	flaskTbl[row]:SetPoint("TOPLEFT", 140, -2)
	flaskTbl[row]:SetWidth(RC.FlFoRuWidth)
	flaskTbl[row]:SetCallback("OnClick", function(self, button)
		if(IsControlKeyDown() and allowCtrlWhisper) then
			RC:Whisper(row);
		end
	end)
	playerBar[row]:AddChild(flaskTbl[row])

	foStatus, foColor = RC:SetFlFoRu(row, 4)
	foodTbl[row] = RC_UI:Create("InteractiveLabel")
	foodTbl[row]:SetText(foStatus)
	foodTbl[row]:SetColor(unpack(foColor))
	foodTbl[row]:SetPoint("TOPLEFT", 200, -2)
	foodTbl[row]:SetWidth(RC.FlFoRuWidth)
	foodTbl[row]:SetCallback("OnClick", function(self, button)
		if(IsControlKeyDown() and allowCtrlWhisper) then
			RC:Whisper(row);
		end
	end)
	playerBar[row]:AddChild(foodTbl[row])

	if(self.db.char.Show.Rune) then
		ruStatus, ruColor = RC:SetFlFoRu(row, 5)
		runeTbl[row] = RC_UI:Create("InteractiveLabel")
		runeTbl[row]:SetText(ruStatus)
		runeTbl[row]:SetColor(unpack(ruColor))
		runeTbl[row]:SetPoint("TOPLEFT", 260, -2)
		runeTbl[row]:SetWidth(RC.FlFoRuWidth)
		runeTbl[row]:SetCallback("OnClick", function(self, button)
		if(IsControlKeyDown() and allowCtrlWhisper) then
			RC:Whisper(row);
		end
	end)
		playerBar[row]:AddChild(runeTbl[row])
	end

	if RC.VantusRuneCheckEnabled then
		vaStatus, vaColor = RC:SetFlFoRu(row, 8)
		vantusruneTbl[row] = RC_UI:Create("InteractiveLabel")
		vantusruneTbl[row]:SetText(vaStatus)
		vantusruneTbl[row]:SetColor(unpack(vaColor))
		vantusruneTbl[row]:SetPoint("TOPLEFT", 260, -2)
		vantusruneTbl[row]:SetWidth(RC.FlFoRuWidth)
		vantusruneTbl[row]:SetCallback("OnClick", function(self, button)
		if(IsControlKeyDown() and allowCtrlWhisper) then
			RC:Whisper(row);
		end
	end)
		playerBar[row]:AddChild(vantusruneTbl[row])
	end

	scroll:AddChild(playerBar[row])
end

function RC:updateGUI()
	local numPlayers = GetNumGroupMembers()
	if numPlayers == 0 then
		numPlayers = 1
	end

	-- If RC is open and a player leaves, when using refresh we need to ensure we are removing the extra row
	local numRows = table.getn(playerBar)
	if numPlayers < numRows then
		local extraRow = numRows - numPlayers
		for j = 1, extraRow do
			table.remove(playerBar)  -- without passing a position to the remove function it just removes the last element in the array
		end
	end

	for i = 1, numPlayers do
		_, _, subgroup, _, _, _, _, _, _, _, _ = GetRaidRosterInfo(i);
		if not ((subgroup == 7 or subgroup == 8) and self.db.char.IgnoreGroup7_8) then
			if not playerBar[i] then
				local posY = -2-(15*(i-1))
				RC:PlayerRow(posY, i)
			end

			if(RC.playerData[i]) then
				RC:SetPlayerText(i);
			end
		end
	end
end

function RC:SetPlayerText(barNr)
	local flStatus, flColor, foStatus, foColor, ruStatus, ruColor
	flStatus, flColor = RC:SetFlFoRu(barNr, 3)
	foStatus, foColor = RC:SetFlFoRu(barNr, 4)
	ruStatus, ruColor = RC:SetFlFoRu(barNr, 5)

	nameTbl[barNr]:SetText(RC:PlayerName(barNr));
	nameTbl[barNr]:SetColor(unpack(RC:SetClassColor(barNr)))

	--Update Flask
	flaskTbl[barNr]:SetText(flStatus)
	flaskTbl[barNr]:SetColor(unpack(flColor))

	--Update Food
	foodTbl[barNr]:SetText(foStatus)
	foodTbl[barNr]:SetColor(unpack(foColor))

	--Update Rune
	if(self.db.char.Show.Rune) then
		runeTbl[barNr]:SetText(ruStatus)
		runeTbl[barNr]:SetColor(unpack(ruColor))
	end
end

function RC:SettingsTab(container)
	-- Flask Group
	local flaskGroup = RC_UI:Create("InlineGroup")
	flaskGroup:SetTitle(L["flask"])
	flaskGroup:SetWidth(RC.GrpWidth)

	local flaskDD = RC_UI:Create("Dropdown")
	flaskDD:SetList(RC.FlaskLevels)
	flaskDD:SetValue(self.db.char.Threshold.Flask)
	flaskDD:SetCallback("OnValueChanged", function(_, _, value)
		self.db.char.Threshold.Flask = value;
	end)
	flaskDD:SetWidth(RC.DDWidth)
	flaskGroup:AddChild(flaskDD)

	local announceFlask = RC_UI:Create("CheckBox")
	announceFlask:SetLabel(L["annFlask"])
	announceFlask:SetType("checkbox")
	announceFlask:SetValue(self.db.char.Announce.Flask)
	announceFlask:SetCallback("OnValueChanged", function(_, _, value)
		self.db.char.Announce.Flask = value;
	end)
	flaskGroup:AddChild(announceFlask)

	local felFocuser = RC_UI:Create("CheckBox")
	felFocuser:SetLabel(L["allowFelFocuser"])
	felFocuser:SetType("checkbox")
	felFocuser:SetValue(self.db.char.Allow.FelFocuser)
	felFocuser:SetCallback("OnValueChanged", function(_, _, value)
		self.db.char.Allow.FelFocuser = value;
	end)
	flaskGroup:AddChild(felFocuser)

	-- Food Group
	local foodGroup = RC_UI:Create("InlineGroup")
	foodGroup:SetTitle(L["food"])
	foodGroup:SetWidth(RC.GrpWidth)

	local foodDD = RC_UI:Create("Dropdown")
	foodDD:SetList(RC.FoodLevels)
	foodDD:SetValue(self.db.char.Threshold.Food)
	foodDD:SetCallback("OnValueChanged", function(_, _, value)
		self.db.char.Threshold.Food = value;
	end)
	foodDD:SetWidth(RC.DDWidth)
	foodGroup:AddChild(foodDD)

	local announceFood = RC_UI:Create("CheckBox")
	announceFood:SetLabel(L["annFood"])
	announceFood:SetType("checkbox")
	announceFood:SetValue(self.db.char.Announce.Food)
	announceFood:SetCallback("OnValueChanged", function(_, _, value)
		self.db.char.Announce.Food = value;
	end)
	foodGroup:AddChild(announceFood)

	-- Rune Group
	local runeGroup = RC_UI:Create("InlineGroup")
	runeGroup:SetTitle(L["rune"])
	runeGroup:SetWidth(RC.GrpWidth)

	local runeDD = RC_UI:Create("Dropdown")
	runeDD:SetList(RC.RuneLevels)
	runeDD:SetValue(self.db.char.Threshold.Rune)
	runeDD:SetCallback("OnValueChanged", function(_, _, value)
		self.db.char.Threshold.Rune = value;
	end)
	runeDD:SetWidth(RC.DDWidth)
	runeGroup:AddChild(runeDD)

	local runeCb = RC_UI:Create("CheckBox")
	runeCb:SetLabel(L["enRune"])
	runeCb:SetType("checkbox")
	runeCb:SetValue(self.db.char.Show.Rune)
	runeCb:SetCallback("OnValueChanged", function(_, _, value)
		self.db.char.Show.Rune = value;
	end)
	runeGroup:AddChild(runeCb)

	local announceRune = RC_UI:Create("CheckBox")
	announceRune:SetLabel(L["annRunes"])
	announceRune:SetType("checkbox")
	announceRune:SetValue(self.db.char.Announce.Rune)
	announceRune:SetCallback("OnValueChanged", function(_, _, value)
		self.db.char.Announce.Rune = value;
	end)
	announceRune:SetCallback("OnEnter", function()
		GameTooltip:SetOwner(UIParent, "ANCHOR_CURSOR");
		GameTooltip:SetText(L["annRunes"]);
		GameTooltip:Show();
	end)
	announceRune:SetCallback("OnLeave", function(self)
		GameTooltip:Hide()
	end)
	runeGroup:AddChild(announceRune)

	-- Vantus Rune
	local vantusGroup = RC_UI:Create("InlineGroup")
	vantusGroup:SetTitle(L["vantusrune"])
	vantusGroup:SetWidth(RC.GrpWidth)

	local vantusDD = RC_UI:Create("Dropdown")
	vantusDD:SetList(RC.VantusBossNames)
	vantusDD:SetValue(self.db.char.Threshold.VantusRune)
	vantusDD:SetCallback("OnValueChanged", function(_, _, value)
		self.db.char.Threshold.VantusRune = value;
	end)
	vantusDD:SetWidth(RC.DDWidth*2)
	vantusGroup:AddChild(vantusDD)

	local whispVantus = RC_UI:Create("CheckBox")
	whispVantus:SetLabel(L["whispVantus"])
	whispVantus:SetType("checkbox")
	whispVantus:SetValue(self.db.char.Whisper.VantusRune)
	whispVantus:SetCallback("OnValueChanged", function(_, _, value)
		self.db.char.Whisper.VantusRune = value;
	end)
	whispVantus:SetCallback("OnEnter", function()
		GameTooltip:SetOwner(UIParent, "ANCHOR_CURSOR");
		GameTooltip:SetText(L["whispVantus"]);
		GameTooltip:Show();
	end)
	whispVantus:SetCallback("OnLeave", function(self)
		GameTooltip:Hide()
	end)
	vantusGroup:AddChild(whispVantus)

	-- Class Raid Buff
	local classGroup = RC_UI:Create("InlineGroup")
	classGroup:SetTitle(L["class"])
	classGroup:SetWidth(RC.GrpWidth)

	local announceClass = RC_UI:Create("CheckBox")
	announceClass:SetLabel(L["annClass"])
	announceClass:SetType("checkbox")
	announceClass:SetValue(self.db.char.Announce.ClassBuffs)
	announceClass:SetCallback("OnValueChanged", function(_, _, value)
		self.db.char.Announce.ClassBuffs = value;
	end)
	classGroup:AddChild(announceClass)

	local enableStat = RC_UI:Create("CheckBox")
	enableStat:SetLabel(L["enStat"])
	enableStat:SetType("checkbox")
	enableStat:SetValue(self.db.char.Allow.PrimaryStat)
	enableStat:SetCallback("OnValueChanged", function(_, _, value)
		self.db.char.Allow.PrimaryStat = value;
	end)
	classGroup:AddChild(enableStat)

	local enableDmgDebuff = RC_UI:Create("CheckBox")
	enableDmgDebuff:SetLabel(L["enDmgDebuff"])
	enableDmgDebuff:SetType("checkbox")
	enableDmgDebuff:SetValue(self.db.char.Allow.DmgModDeBuff)
	enableDmgDebuff:SetCallback("OnValueChanged", function(_, _, value)
		self.db.char.Allow.DmgModDeBuff = value;
	end)
	enableDmgDebuff:SetCallback("OnEnter", function()
		GameTooltip:SetOwner(UIParent, "ANCHOR_CURSOR");
		GameTooltip:SetText(L["enDmgDebuff"]);
		GameTooltip:Show();
	end)
	enableDmgDebuff:SetCallback("OnLeave", function(self)
		GameTooltip:Hide();
	end)
	classGroup:AddChild(enableDmgDebuff)

	local enableUtility = RC_UI:Create("CheckBox")
	enableUtility:SetLabel(L["enUtility"])
	enableUtility:SetType("checkbox")
	enableUtility:SetValue(self.db.char.Allow.Utility)
	enableUtility:SetCallback("OnValueChanged", function(_, _, value)
		self.db.char.Allow.Utility = value;
	end)
	classGroup:AddChild(enableUtility)

	local partialUtil = RC_UI:Create("CheckBox")
	partialUtil:SetLabel(L["partUtil"])
	partialUtil:SetType("checkbox")
	partialUtil:SetValue(self.db.char.Announce.UtilityPart)
	partialUtil:SetCallback("OnValueChanged", function(_, _, value)
		self.db.char.Announce.UtilityPart = value;
	end)
	partialUtil:SetCallback("OnEnter", function()
		GameTooltip:SetOwner(UIParent, "ANCHOR_CURSOR");
		GameTooltip:SetText(L["PartUtilTT"]);
		GameTooltip:Show();
	end)
	partialUtil:SetCallback("OnLeave", function(self)
		GameTooltip:Hide();
	end)
	classGroup:AddChild(partialUtil)

	-- Behavior
	local behavGroup = RC_UI:Create("InlineGroup")
	behavGroup:SetTitle(L["Beha"])
	behavGroup:SetWidth(RC.GrpWidth)

	local rdyCheckCb = RC_UI:Create("CheckBox")
	rdyCheckCb:SetLabel(L["rdyCheck"])
	rdyCheckCb:SetType("checkbox")
	rdyCheckCb:SetValue(self.db.char.Show.RaidCheckonReady)
	rdyCheckCb:SetCallback("OnValueChanged", function(_, _, value)
		self.db.char.Show.RaidCheckonReady = value;
	end)
	behavGroup:AddChild(rdyCheckCb)

	local annRdyCheckCb = RC_UI:Create("CheckBox")
	annRdyCheckCb:SetLabel(L["rdyAnn"])
	annRdyCheckCb:SetType("checkbox")
	annRdyCheckCb:SetValue(self.db.char.Announce.ReadyCheck)
	annRdyCheckCb:SetCallback("OnValueChanged", function(_, _, value)
		self.db.char.Announce.ReadyCheck = value;
	end)
	behavGroup:AddChild(annRdyCheckCb)

	local whispRdyCheckCb = RC_UI:Create("CheckBox")
	whispRdyCheckCb:SetLabel(L["rdyWhi"])
	whispRdyCheckCb:SetType("checkbox")
	whispRdyCheckCb:SetValue(self.db.char.Whisper.OnReadyCheck)
	whispRdyCheckCb:SetCallback("OnValueChanged", function(_, _, value)
		self.db.char.Whisper.OnReadyCheck = value;
	end)
	behavGroup:AddChild(whispRdyCheckCb)

	local truncateChatCb = RC_UI:Create("CheckBox")
	truncateChatCb:SetLabel(L["truncatechat"])
	truncateChatCb:SetType("checkbox")
	truncateChatCb:SetValue(self.db.char.TruncateChat)
	truncateChatCb:SetCallback("OnValueChanged", function(_, _, value)
		self.db.char.TruncateChat = value;
	end)
	truncateChatCb:SetCallback("OnEnter", function()
		GameTooltip:SetOwner(UIParent, "ANCHOR_CURSOR");
		GameTooltip:SetText(L["truncateTT"]);
		GameTooltip:Show();
	end)
	truncateChatCb:SetCallback("OnLeave", function(self)
		GameTooltip:Hide()
	end)
	behavGroup:AddChild(truncateChatCb)

	local combatHideCb = RC_UI:Create("CheckBox")
	combatHideCb:SetLabel(L["hideCombat"])
	combatHideCb:SetType("checkbox")
	combatHideCb:SetValue(self.db.char.Show.StartCombatHide)
	combatHideCb:SetCallback("OnValueChanged", function(_, _, value)
		self.db.char.Show.StartCombatHide = value;
	end)
	behavGroup:AddChild(combatHideCb)

	local ldrAsstOnlyCb = RC_UI:Create("CheckBox")
	ldrAsstOnlyCb:SetLabel(L["LeaderAssistOnly"])
	ldrAsstOnlyCb:SetType("checkbox")
	ldrAsstOnlyCb:SetValue(self.db.char.Announce.LeaderAssistOnly)
	ldrAsstOnlyCb:SetCallback("OnValueChanged", function(_, _, value)
		self.db.char.Announce.LeaderAssistOnly = value
	end)
	ldrAsstOnlyCb:SetCallback("OnEnter", function()
		GameTooltip:SetOwner(UIParent, "ANCHOR_CURSOR");
		GameTooltip:SetText(L["LeaderAssistOnly"]);
		GameTooltip:Show();
	end)
	ldrAsstOnlyCb:SetCallback("OnLeave", function(self)
		GameTooltip:Hide()
	end)
	behavGroup:AddChild(ldrAsstOnlyCb)

	local annBossStartCb = RC_UI:Create("CheckBox")
	annBossStartCb:SetLabel(L["annBossStart"])
	annBossStartCb:SetType("checkbox")
	annBossStartCb:SetValue(self.db.char.Announce.BossStart)
	annBossStartCb:SetCallback("OnValueChanged", function(_, _, value)
		self.db.char.Announce.BossStart = value
	end)
	annBossStartCb:SetCallback("OnEnter", function()
		GameTooltip:SetOwner(UIParent, "ANCHOR_CURSOR");
		GameTooltip:SetText(L["annBossStart"]);
		GameTooltip:Show();
	end)
	annBossStartCb:SetCallback("OnLeave", function(self)
		GameTooltip:Hide()
	end)
	behavGroup:AddChild(annBossStartCb)

	local showRealms = RC_UI:Create("CheckBox")
	showRealms:SetLabel(L["showRealms"])
	showRealms:SetType("checkbox")
	showRealms:SetValue(self.db.char.Show.Realm)
	showRealms:SetCallback("OnValueChanged", function(_, _, value)
		self.db.char.Show.Realm = value;
	end)
	behavGroup:AddChild(showRealms)

	local ign78Cb = RC_UI:Create("CheckBox")
	ign78Cb:SetLabel(L["ign78"])
	ign78Cb:SetType("checkbox")
	ign78Cb:SetValue(self.db.char.IgnoreGroup7_8)
	ign78Cb:SetCallback("OnValueChanged", function(_, _, value)
		self.db.char.IgnoreGroup7_8 = value;
	end)
	behavGroup:AddChild(ign78Cb)

	-- UI Group
	local uiGroup = RC_UI:Create("InlineGroup")
	uiGroup:SetTitle(L["UI"])
	uiGroup:SetWidth(RC.GrpWidth)

	local miniMapCb = RC_UI:Create("CheckBox")
	miniMapCb:SetLabel(L["showMinimap"])
	miniMapCb:SetType("checkbox")
	miniMapCb:SetValue(not self.db.char.Show.Minimap.hide)
	miniMapCb:SetCallback("OnValueChanged", function(_, _, value)
		self.db.char.Show.Minimap.hide = not value;
		RaidCheck:UpdateMinimap();
	end)
	uiGroup:AddChild(miniMapCb)

	local classColors = RC_UI:Create("CheckBox")
	classColors:SetLabel(L["colorClass"])
	classColors:SetType("checkbox")
	classColors:SetValue(self.db.char.classColors)
	classColors:SetCallback("OnValueChanged", function(_, _, value)
		self.db.char.classColors = value;
	end)
	uiGroup:AddChild(classColors)

	local ctrlWhisp = RC_UI:Create("CheckBox")
	ctrlWhisp:SetLabel(L["ctrlWhi"])
	ctrlWhisp:SetType("checkbox")
	ctrlWhisp:SetValue(self.db.char.Whisper.OnControl)
	ctrlWhisp:SetCallback("OnValueChanged", function(_, _, value)
		self.db.char.Whisper.OnControl = value;
	end)
	ctrlWhisp:SetCallback("OnEnter", function()
		GameTooltip:SetOwner(UIParent, "ANCHOR_CURSOR");
		GameTooltip:SetText(L["ctrlWhi"]);
		GameTooltip:Show();
	end)
	ctrlWhisp:SetCallback("OnLeave", function(self)
		GameTooltip:Hide()
	end)
	uiGroup:AddChild(ctrlWhisp)

	local mainWindowHeight = RC_UI:Create("Slider")
	mainWindowHeight:SetLabel(L["windowSizeSlider"])
	mainWindowHeight:SetSliderValues(RC.MinHeight, RC.MaxHeight, RC.SliderStep) -- Minimum, Maximum, Step value
	mainWindowHeight:SetValue(self.db.char.Frame.height)
	mainWindowHeight:SetCallback("OnMouseUp", function(_, _, value)
		self.db.char.Frame.height = value;
		RC_Frame:SetStatusTable(self.db.char.Frame)
		RC_Frame:ApplyStatus()
	end)
	uiGroup:AddChild(mainWindowHeight)

	local resetSettingsDefault = RC_UI:Create("Button")
	resetSettingsDefault:SetText(L["resetDefault"])
	resetSettingsDefault:SetCallback("OnClick", function ()
		RC:UIresettoDefault()
	end)
	uiGroup:AddChild(resetSettingsDefault)

	local scrollcontainer = RC_UI:Create("SimpleGroup") -- "InlineGroup" is also good
	scrollcontainer:SetFullWidth(true)
	scrollcontainer:SetFullHeight(true)
	scrollcontainer:SetLayout("Fill")

	local scroll = RC_UI:Create("ScrollFrame")
	scroll:SetLayout("Flow")
	scroll:AddChild(flaskGroup)
	scroll:AddChild(foodGroup)
	scroll:AddChild(runeGroup)
	scroll:AddChild(vantusGroup)
	scroll:AddChild(classGroup)
	scroll:AddChild(behavGroup)
	scroll:AddChild(uiGroup)
	scrollcontainer:AddChild(scroll)

	container:AddChild(scrollcontainer)
end

-- Callback function for OnGroupSelected
local function SelectGroup(container, event, group)
   container:ReleaseChildren()
   if group == "RC_Main" then
      RC:MainTab(container)
   elseif group == "RC_Settings" then
      RC:SettingsTab(container)
   end
end

function RC:MainFrame(opentotab)
	if opentotab then
		_activeTab = opentotab
	end

	-- Create the frame container
	RC_Frame = RC_UI:Create("Window")
	RC_Frame:SetTitle(RC.AddOn_Name.."  [Version: "..GetAddOnMetadata("RaidCheck", "Version").."]")
	if floor(self.db.char.Frame.width) ~= RC.MainFrameWidth then self.db.char.Frame.width = RC.MainFrameWidth end;
	RC_Frame:SetStatusTable(self.db.char.Frame)
	RC_Frame:ApplyStatus()
	RC_Frame:EnableResize(false)

	-- Fill Layout - the TabGroup widget will fill the whole frame
	RC_Frame:SetLayout("Fill")

	RC_Tabs = RC_UI:Create("TabGroup")
	RC_Tabs:SetLayout("Flow")
	-- Setup which tabs to show
	RC_Tabs:SetTabs({{text=L["main"], value="RC_Main"}, {text=L["settings"], value="RC_Settings"}})
	-- Register callback
	RC_Tabs:SetCallback("OnGroupSelected", SelectGroup)
	-- Set initial Tab (this will fire the OnGroupSelected callback)
	RC_Tabs:SetFullWidth(true)
	RC_Tabs:SelectTab(_activeTab)

	RC_Frame:AddChild(RC_Tabs)
end

-- toggle visibility of the UI
function RC:Toggle(tab)
	if not self:IsEnabled() then return end

	if not tab then
		tab = _activeTab
	end

	if not RC_Frame then
		self:Show(tab)
	elseif RC_Frame:IsShown() then
		self:Hide()
	else
		self:Show(tab)
	end
end

-- hide the UI if it is currently showing
function RC:Hide()
	if ((not self:IsEnabled()) or (not RC_Frame)) then return end

	RC_Frame:Hide()
end

-- show the UI if it is not currently showing
function RC:Show(tab)
	if not self:IsEnabled() then
		return
	elseif RC_Frame then
		if RC_Frame:IsShown() then
			return
		end
	end

	if not tab then
		RC:MainFrame("RC_Main")
	elseif tab == "RC_Main" or tab == "RC_Settings" then
		RC:MainFrame(tab)
	end
end

function RC:OpenMain()
	RC:Toggle("RC_Main")
end

function RC:OpenSettings()
	RC:Toggle("RC_Settings")
end
